
plm.rank.data.setup <- function(x,y,delta) {
  	x <- as.matrix(x)
  	dimnum <- dim(x)
  	n <- dimnum[1]
  	p <- dimnum[2]
  	
  	yy0 <- rep(y,rep(n,n))
  	delta1 <- rep(delta,rep(n,n))
  	yy1 <- rep(y,n)
  	yy <- delta1*(yy0-yy1)
  	xx0 <- matrix(rep(as.vector(x),rep(n,n*p)),nrow=n*n)
  	xx1 <- t(matrix(rep(as.vector(t(x)),n),nrow=p))
  	xx <- xx0-xx1   
  	xxdif <- xx*delta1 
	invisible(list(x=xxdif,y=yy))	
	}
	
loc.setupfun <- function(J,S,x,y,delta) {
	flag <- S==J
	xr <- x[flag,]
	yr <- y[flag]
	dr <- delta[flag]
	tmp <- plm.rank.data.setup(x=xr,y=yr,delta=dr)
	t(cbind(tmp$y,tmp$x,S[flag]))
	}

# INPUT 
#	x = covariate matrix
#	w = covariate scalar
#	y = survival time
# 	delta = failure indicator
#	M = number of strata
#	S = (optional) strata levels: 1, 2, ...
# OUTPUT
#  	bhat = vector of regression coefficients

plm.rank.l1 <- function(x,w,y,delta,lambda,M=3,S=NULL) {
	if(!any(search()=="package:quantreg"))
    stop("plm.rank:  The 'quantreg' package is not loaded.")

	x <- as.matrix(x)
	
	dimnum <- dim(x)
	n <- dimnum[1]
	p <- dimnum[2]
	
	#cat("n = ",n,"\t and p = ",p,"\n")
	
	if(is.null(S)) {
		q <- quantile(w,probs=seq(0,1,length=(M+1)),names=FALSE)
		S <- cut(w,breaks=q,include.lowest=TRUE,right=TRUE)
		}
	else {
		if(min(S)<1) stop("S cannot be less than 1.")
		}
	Snew <- as.numeric(S)
	myrows <- sum(tapply(Snew,list(Snew),FUN=function(x){length(x)^2}))
	mycols <- p + 2 # (y,x,S)
	tmpapply <- lapply(1:max(Snew),loc.setupfun,Snew,x,y,delta)
	dat <- matrix(unlist(tmpapply),nrow=myrows,ncol=mycols,byrow=TRUE)
	
	yy <- dat[,1]
	xx <- dat[,2:(p+1)]
	yLARGE <- 1000*(n)^2
	
	if (p>1) xsum <- apply(xx,2,sum)
	else xsum <- sum(xx)
	
	yynew <- c(yy,yLARGE,rep(0,p))
	
	if (p>1) xxnew <- rbind(xx,-xsum,diag(rep(n*n*lambda,p)))
	else xxnew <- c(xx,-xsum,n*n*lambda)
	
	fit <- rq.fit.fnb(xxnew,yynew,tau=0.5)
	as.vector(fit$coef)
	
	}
	
gehan.loss <- function(x,y,delta,beta) {
	res <- y - as.vector(x %*% beta)
	tmp.fun <- function(I,res,del) {
		if(del[I] < 0.5) out <- 0
		else {
			ei <- res[I]
			diff <- ei - res
			out <- -1 * sum(diff[diff <= 0])
			}
		out/length(res)
		}
	vec <- unlist(lapply(1:length(y),tmp.fun,res,delta))
	sum(vec)
	}

strat.loss <- function(x,S,y,delta,beta) {
	Slevels <- 1:max(S)
	wrap.fun <- function(J,S,x,y,delta,beta) {
		index <- S==J
		xin <- x[index,]
		yin <- y[index]
		din <- delta[index]
		Lg <- gehan.loss(xin,yin,din,beta)
		}
	Lall <- unlist(lapply(Slevels,wrap.fun,S,x,y,delta,beta))
	sum(Lall)
	}

local.cvfun <- function(LVEC,x.tr,S.tr,y.tr,del.tr,M,x.te,S.te,y.te,del.te) {
	
	tmp.cvfun <- function(J,LVEC,x.tr,S.tr,y.tr,del.tr,M,x.te,S.te,y.te,del.te) {
		LAMBDA <- LVEC[J]
		w.tr <- rep(0,length(y.tr))
		bhat.tr <- plm.rank.l1(x.tr,w.tr,y.tr,del.tr,lambda=LAMBDA,M=M,S=S.tr)
		#cat("b.tr = ",round(bhat.tr,2),"\n")
		lss <- strat.loss(x.te,S.te,y.te,del.te,bhat.tr)
		#cat("lss = ",round(lss,3),"\n")
		lss
		}
	loc.out <- unlist(lapply(1:length(LVEC),tmp.cvfun,LVEC,x.tr,S.tr,y.tr,del.tr,M,x.te,S.te,y.te,del.te))
	loc.out
	}

cv.plm.rank.l1 <- function(x,w,y,delta,lamvec,M=3,S=NULL,V=10) {
	if(!any(search()=="package:quantreg"))
    stop("plm.rank:  The 'quantreg' package is not loaded.")

	if(is.null(S)) {
		q <- quantile(w,probs=seq(0,1,length=(M+1)),names=FALSE)
		S <- cut(w,breaks=q,include.lowest=TRUE,right=TRUE)
		}
	else {
		if(min(S)<1) stop("S cannot be less than 1.")
		}
	Snew <- as.numeric(S)

	n <- length(y)
	s <- sample(1:n,n,replace=FALSE)
	
	LOSS.MAT <- matrix(numeric(0),V,length(lamvec))
	for (J in 1:V) {
		cat("J = ",J,"\n")
		te.ind <- s[(1+(J-1)*floor(n/V)):(J*floor(n/V))]
		x.tr <- x[-te.ind,]
		S.tr <- Snew[-te.ind]
		y.tr <- y[-te.ind]
		del.tr <- delta[-te.ind]
		x.te <- x[te.ind,]
		S.te <- Snew[te.ind]
		y.te <- y[te.ind]
		del.te <- delta[te.ind]
		LOSS.MAT[J,] <- local.cvfun(lamvec,x.tr,S.tr,y.tr,del.tr,M,x.te,S.te,y.te,del.te)
		}
	
	lvec <- apply(LOSS.MAT,2,mean)
	sdvec <- apply(LOSS.MAT,2,sd)
	out <- cbind(lamvec,lvec,sdvec)
	m.cv <- lamvec[order(lvec)[1]]
	list(out=out,mcv=m.cv)
	
	}
	